@@ -3,10 +3,11 @@ package App::Netdisco::Core::Arpnip;
 use Dancer qw/:syntax :script/;
 use Dancer::Plugin::DBIC 'schema';
 
-use App::Netdisco::Util::SanityCheck 'check_mac';
+use App::Netdisco::Util::Node 'check_mac';
 use App::Netdisco::Util::DNS ':all';
 use NetAddr::IP::Lite ':lower';
 use Time::HiRes 'gettimeofday';
+use NetAddr::MAC ();
 
 use base 'Exporter';
 our @EXPORT = ();
@@ -96,14 +97,21 @@ sub _get_arps {
   return $resolved_ips;
 }
 
-=head2 store_arp( $mac, $ip, $name, $now? )
+=head2 store_arp( \%host, $now? )
 
 Stores a new entry to the C<node_ip> table with the given MAC, IP (v4 or v6)
-and DNS host name.
+and DNS host name. Host details are provided in a Hash ref:
 
-Will mark old entries for this IP as no longer C<active>.
+ {
+    ip   => '192.0.2.1',
+    node => '00:11:22:33:44:55',
+    dns  => 'myhost.example.com',
+ }
 
-Optionally a literal string can be passed in the fourth argument for the
+The C<dns> entry is optional. The update will mark old entries for this IP as
+no longer C<active>.
+
+Optionally a literal string can be passed in the second argument for the
 C<time_last> timestamp, otherwise the current timestamp (C<now()>) is used.
 
 =cut
@@ -112,9 +120,11 @@ sub store_arp {
   my ($hash_ref, $now) = @_;
   $now ||= 'now()';
   my $ip   = $hash_ref->{'ip'};
-  my $mac  = $hash_ref->{'node'};
+  my $mac  = NetAddr::MAC->new($hash_ref->{'node'});
   my $name = $hash_ref->{'dns'};
 
+  return if !defined $mac or $mac->errstr;
+
   schema('netdisco')->txn_do(sub {
     my $current = schema('netdisco')->resultset('NodeIp')
       ->search(
@@ -124,7 +134,7 @@ sub store_arp {
     schema('netdisco')->resultset('NodeIp')
       ->update_or_create(
       {
-        mac => $mac,
+        mac => $mac->as_ieee,
         ip => $ip,
         dns => $name,
         active => \'true',
