@@ -1,10 +1,12 @@
 package App::Netdisco::Util::SNMP;
 
 use Dancer qw/:syntax :script/;
-use App::Netdisco::Util::Device qw/get_device check_acl check_no/;
+use App::Netdisco::Util::Device 'get_device';
+use App::Netdisco::Util::Permission qw/check_acl_no check_acl/;
 
 use SNMP::Info;
 use Try::Tiny;
+use Module::Load ();
 use Path::Class 'dir';
 
 use base 'Exporter';
@@ -41,7 +43,7 @@ Returns C<undef> if the connection fails.
 
 =cut
 
-sub snmp_connect { _snmp_connect_generic(@_, 'read') }
+sub snmp_connect { _snmp_connect_generic('read', @_) }
 
 =head2 snmp_connect_rw( $ip )
 
@@ -52,18 +54,21 @@ Returns C<undef> if the connection fails.
 
 =cut
 
-sub snmp_connect_rw { _snmp_connect_generic(@_, 'write') }
+sub snmp_connect_rw { _snmp_connect_generic('write', @_) }
 
 sub _snmp_connect_generic {
-  my ($ip, $mode) = @_;
+  my ($mode, $ip, $useclass) = @_;
   $mode ||= 'read';
 
   # get device details from db
   my $device = get_device($ip);
 
   my %snmp_args = (
+    AutoSpecify => 0,
     DestHost => $device->ip,
-    Retries => (setting('snmpretries') || 2),
+    # 0 is falsy. Using || with snmpretries equal to 0 will set retries to 2.
+    # check if the setting is 0. If not, use the default value of 2.
+    Retries => (setting('snmpretries') || setting('snmpretries') == 0 ? 0 : 2),
     Timeout => (setting('snmptimeout') || 1000000),
     NonIncreasing => (setting('nonincreasing') || 0),
     BulkWalk => ((defined setting('bulkwalk_off') && setting('bulkwalk_off'))
@@ -72,10 +77,11 @@ sub _snmp_connect_generic {
     MibDirs => [ _build_mibdirs() ],
     IgnoreNetSNMPConf => 1,
     Debug => ($ENV{INFO_TRACE} || 0),
+    DebugSNMP => ($ENV{SNMP_TRACE} || 0),
   );
 
   # an override for bulkwalk
-  $snmp_args{BulkWalk} = 0 if check_no($device, 'bulkwalk_no');
+  $snmp_args{BulkWalk} = 0 if check_acl_no($device, 'bulkwalk_no');
 
   # further protect against buggy Net-SNMP, and disable bulkwalk
   if ($snmp_args{BulkWalk}
@@ -92,19 +98,16 @@ sub _snmp_connect_generic {
 
   # which SNMP versions to try and in what order
   my @versions =
-    ( check_no($device->ip, 'snmpforce_v3') ? (3)
-    : check_no($device->ip, 'snmpforce_v2') ? (2)
-    : check_no($device->ip, 'snmpforce_v1') ? (1)
+    ( check_acl_no($device->ip, 'snmpforce_v3') ? (3)
+    : check_acl_no($device->ip, 'snmpforce_v2') ? (2)
+    : check_acl_no($device->ip, 'snmpforce_v1') ? (1)
     : (reverse (1 .. (setting('snmpver') || 3))) );
 
   # use existing or new device class
-  my @classes = ('SNMP::Info');
-  if ($device->snmp_class) {
+  my @classes = ($useclass || 'SNMP::Info');
+  if ($device->snmp_class and not $useclass) {
       unshift @classes, $device->snmp_class;
   }
-  else {
-      $snmp_args{AutoSpecity} = 1;
-  }
 
   my $info = undef;
   COMMUNITY: foreach my $comm (@communities) {
@@ -120,7 +123,8 @@ sub _snmp_connect_generic {
               next unless $class;
 
               my %local_args = (%snmp_args, Version => $ver);
-              $info = _try_connect($device, $class, $comm, $mode, \%local_args);
+              $info = _try_connect($device, $class, $comm, $mode, \%local_args,
+                ($useclass ? 0 : 1) );
               last COMMUNITY if $info;
           }
       }
@@ -130,30 +134,31 @@ sub _snmp_connect_generic {
 }
 
 sub _try_connect {
-  my ($device, $class, $comm, $mode, $snmp_args) = @_;
+  my ($device, $class, $comm, $mode, $snmp_args, $reclass) = @_;
   my %comm_args = _mk_info_commargs($comm);
+  my $debug_comm = ( $comm->{community}
+      ? $ENV{SHOW_COMMUNITY} ? $comm->{community} : '<hidden>'
+      : "v3user:$comm->{user}" );
   my $info = undef;
 
   try {
       debug
         sprintf '[%s] try_connect with ver: %s, class: %s, comm: %s',
-        $snmp_args->{DestHost}, $snmp_args->{Version}, $class,
-        ($comm->{community} || "v3user:$comm->{user}");
-      eval "require $class";
+        $snmp_args->{DestHost}, $snmp_args->{Version}, $class, $debug_comm;
+      Module::Load::load $class;
 
-      $info = $class->new(%$snmp_args, %comm_args);
+      $info = $class->new(%$snmp_args, %comm_args) or return;
       $info = ($mode eq 'read' ? _try_read($info, $device, $comm)
                                : _try_write($info, $device, $comm));
 
       # first time a device is discovered, re-instantiate into specific class
-      if ($info and $info->device_type ne $class) {
+      if ($reclass and $info and $info->device_type ne $class) {
           $class = $info->device_type;
           debug
             sprintf '[%s] try_connect with ver: %s, new class: %s, comm: %s',
-            $snmp_args->{DestHost}, $snmp_args->{Version}, $class,
-            ($comm->{community} || "v3user:$comm->{user}");
+            $snmp_args->{DestHost}, $snmp_args->{Version}, $class, $debug_comm;
 
-          eval "require $class";
+          Module::Load::load $class;
           $info = $class->new(%$snmp_args, %comm_args);
       }
   }
@@ -186,7 +191,7 @@ sub _try_read {
 
   # regardless of device in storage, save the hint
   $device->update_or_create_related('community',
-    {snmp_auth_tag => $comm->{tag}}) if $comm->{tag};
+    {snmp_auth_tag_read => $comm->{tag}}) if $comm->{tag};
 
   return $info;
 }
@@ -204,7 +209,7 @@ sub _try_write {
 
   # one of these two cols must be set
   $device->update_or_create_related('community', {
-    ($comm->{tag} ? (snmp_auth_tag => $comm->{tag}) : ()),
+    ($comm->{tag} ? (snmp_auth_tag_write => $comm->{tag}) : ()),
     ($comm->{community} ? (snmp_comm_rw => $comm->{community}) : ()),
   });
 
@@ -218,17 +223,22 @@ sub _mk_info_commargs {
   return (Community => $comm->{community})
     if exists $comm->{community};
 
-  my $seclevel = (exists $comm->{auth} ?
-                  (exists $comm->{priv} ? 'authPriv' : 'authNoPriv' )
-                  : 'noAuthNoPriv');
+  my $seclevel =
+    (exists $comm->{auth} ?
+    (exists $comm->{priv} ? 'authPriv' : 'authNoPriv' )
+                          : 'noAuthNoPriv');
 
   return (
-    SecName => $comm->{user},
+    SecName  => $comm->{user},
     SecLevel => $seclevel,
-    AuthProto => uc (eval { $comm->{auth}->{proto} } || 'MD5'),
-    AuthPass  => (eval { $comm->{auth}->{pass} } || ''),
-    PrivProto => uc (eval { $comm->{priv}->{proto} } || 'DES'),
-    PrivPass  => (eval { $comm->{priv}->{pass} } || ''),
+    ( exists $comm->{auth} ? (
+      AuthProto => uc ($comm->{auth}->{proto} || 'MD5'),
+      AuthPass  => ($comm->{auth}->{pass} || ''),
+      ( exists $comm->{priv} ? (
+        PrivProto => uc ($comm->{priv}->{proto} || 'DES'),
+        PrivPass  => ($comm->{priv}->{pass} || ''),
+      ) : ()),
+    ) : ()),
   );
 }
 
@@ -248,9 +258,11 @@ sub _get_mibdirs_content {
 sub _build_communities {
   my ($device, $mode) = @_;
   $mode ||= 'read';
+  my $seen_tags = {}; # for cleaning community table
 
   my $config = (setting('snmp_auth') || []);
-  my $stored_tag = eval { $device->community->snmp_auth_tag };
+  my $tag_name = 'snmp_auth_tag_'. $mode;
+  my $stored_tag = eval { $device->community->$tag_name };
   my $snmp_comm_rw = eval { $device->community->snmp_comm_rw };
   my @communities = ();
 
@@ -279,6 +291,7 @@ sub _build_communities {
 
       # defaults
       $stanza->{tag} ||= $tag;
+      ++$seen_tags->{ $stanza->{tag} };
       $stanza->{read} = 1 if !exists $stanza->{read};
       $stanza->{only} ||= ['any'];
       $stanza->{only} = [$stanza->{only}] if ref '' eq ref $stanza->{only};
@@ -288,7 +301,8 @@ sub _build_communities {
            and !exists $stanza->{community};
 
       if ($stanza->{$mode} and check_acl($device, $stanza->{only})) {
-          if ($stored_tag and $stored_tag eq $stanza->{tag}) {
+          if ($device->in_storage and
+            $stored_tag and $stored_tag eq $stanza->{tag}) {
               # last known-good by tag
               unshift @communities, $stanza
           }
@@ -298,6 +312,11 @@ sub _build_communities {
       }
   }
 
+  # clean the community table of obsolete tags
+  if ($stored_tag and !exists $seen_tags->{ $stored_tag }) {
+      eval { $device->community->update({$tag_name => undef}) };
+  }
+
   # legacy config (note: read strings tried before write)
   if ($mode eq 'read') {
       push @communities, map {{
@@ -330,7 +349,7 @@ sub _get_external_community {
       $cmd =~ s/\%HOST\%/$host/egi;
       $cmd =~ s/\%IP\%/$ip/egi;
 
-      my $result = `$cmd`;
+      my $result = `$cmd`; # BACKTICKS
       return () unless defined $result and length $result;
 
       my @lines = split (m/\n/, $result);
@@ -366,21 +385,28 @@ community indexing, with the given C<$vlan> ID. Works for all SNMP versions.
 
 sub snmp_comm_reindex {
   my ($snmp, $device, $vlan) = @_;
-  my $ver  = $snmp->snmp_ver;
+  my $ver = $snmp->snmp_ver;
 
   if ($ver == 3) {
       my $prefix = '';
       my @comms = _build_communities($device, 'read');
       foreach my $c (@comms) {
           next unless $c->{tag}
-            and $c->{tag} eq (eval { $device->community->snmp_auth_tag } || '');
+            and $c->{tag} eq (eval { $device->community->snmp_auth_tag_read } || '');
           $prefix = $c->{context_prefix} and last;
       }
       $prefix ||= 'vlan-';
+
+      debug
+        sprintf '[%s] reindexing to "%s%s" (ver: %s, class: %s)',
+        $device->ip, $prefix, $vlan, $ver, $snmp->class;
       $snmp->update(Context => ($prefix . $vlan));
   }
   else {
       my $comm = $snmp->snmp_comm;
+
+      debug sprintf '[%s] reindexing to vlan %s (ver: %s, class: %s)',
+        $device->ip, $vlan, $ver, $snmp->class;
       $snmp->update(Community => $comm . '@' . $vlan);
   }
 }
