@@ -10,9 +10,13 @@ use Socket6 (); # to ensure dependency is met
 use HTML::Entities (); # to ensure dependency is met
 use URI::QueryParam (); # part of URI, to add helper methods
 use Path::Class 'dir';
+use Module::Find ();
 use Module::Load ();
 use App::Netdisco::Util::Web 'interval_to_daterange';
 
+# can override splats only by loading first
+Module::Find::usesub 'App::NetdiscoE::Web';
+
 use App::Netdisco::Web::AuthN;
 use App::Netdisco::Web::Static;
 use App::Netdisco::Web::Search;
@@ -48,6 +52,26 @@ if (setting('extra_web_plugins') and ref [] eq ref setting('extra_web_plugins'))
     _load_web_plugins( setting('extra_web_plugins') );
 }
 
+# after plugins are loaded, add our own template path
+push @{ config->{engines}->{netdisco_template_toolkit}->{INCLUDE_PATH} },
+     setting('views');
+
+# any template paths in deployment.yml (should override plugins)
+if (setting('template_paths') and ref [] eq ref setting('template_paths')) {
+    push @{setting('template_paths')},
+         dir(($ENV{NETDISCO_HOME} || $ENV{HOME}), 'nd-site-local', 'share')->stringify
+      if (setting('site_local_files'));
+    unshift @{ config->{engines}->{netdisco_template_toolkit}->{INCLUDE_PATH} },
+      @{setting('template_paths')};
+}
+
+# load cookie key from database
+setting('session_cookie_key' => undef);
+my $sessions = schema('netdisco')->resultset('Session');
+my $skey = $sessions->find({id => 'dancer_session_cookie_key'});
+setting('session_cookie_key' => $skey->get_column('a_session')) if $skey;
+Dancer::Session::Cookie::init(session);
+
 # workaround for https://github.com/PerlDancer/Dancer/issues/935
 hook after_error_render => sub { setting('layout' => 'main') };
 
@@ -68,6 +92,10 @@ hook 'before_template' => sub {
     # create date ranges from within templates
     $tokens->{to_daterange}  = sub { interval_to_daterange(@_) };
 
+    # data structure for DataTables records per page menu
+    $tokens->{table_showrecordsmenu} =
+      to_json( setting('table_showrecordsmenu') );
+
     # fix Plugin Template Variables to be only path+query
     $tokens->{$_} = $tokens->{$_}->path_query
       for qw/search_node search_device device_ports/;
